require "import"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.content.Intent"

-- ❌ अगर service या getEditText() नहीं है, तो एरर दिखाएं
if not service or not service.getEditText() then
    service.asyncSpeak("Sorry! Text Box Not Found!")
    return false
end

local autoPunctuationByHemantSingal = ". "

local recordIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
recordIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "hi-IN")

-- शब्दों को बदलने के लिए टेबल
local wordChangeTable = {
    ["कंट्री स्क्रीन रीडर"] = "कॉमेंट्री स्क्रीन रीडर",
    ["धन्यवाद"] = "ढणयवाद",
    ["उसे"] = "उस",
    ["जॉर्ज"] = "JAWS",
    ["एनवीडीया"] = "NVDA",
    ["अल्पविराम"] = ", ",
    ["प्रश्न चिन्ह"] = "? ",
    ["प्रश्नवाचक"] = "? ",
    ["प्रश्न वाचक"] = "? ",
    ["विस्मय"] = "! ",
    ["न्यूलाइन"] = "\n",
    ["नई लाइन"] = "\n",
    ["न्यू लाइन"] = "\n",
    ["स्पेस"] = " ",
    ["थम्स अप"] = "👍",
    ["रेड हार्ट"] = "❤",
    ["रेड हॉट"] = "❤",
    ["फोल्डेड हैंड्स"] = "🙏",
    ["अति सुंदर"] = "👌",
    ["डॉट डॉट"] = "...",
    ["स्माइली फेस"] = "☺"
}

-- स्पीच टेक्स्ट को प्रोसेस करने का फंक्शन
local function processSpeech(spokenText)
    for k, v in pairs(wordChangeTable) do
        spokenText = spokenText:gsub(k, v)
    end
    return spokenText
end

-- स्पीच रिकग्नाइज़र बनाएं और एरर हैंडल करें
local speechRecord = SpeechRecognizer.createSpeechRecognizer(service)
if not speechRecord then
    service.asyncSpeak("Speech recognizer creation failed!")
    return false
end

-- स्पीच रिकग्निशन के लिए लिस्नर सेट करें
speechRecord.setRecognitionListener{
    onResults = function(results)
        local spokenText = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION).get(0)
        spokenText = processSpeech(spokenText)

        -- अगर पहले से पंक्चुएशन नहीं है, तो ऐड करें
        if not spokenText:match("[%.!?]$") then
            spokenText = spokenText .. autoPunctuationByHemantSingal
        end
        
        -- टेक्स्ट बॉक्स में इनपुट करें और बोलें
        service.insertText(service.getEditText(), spokenText)
        service.speak(spokenText)

        -- स्पीच रिकग्नाइज़र को डेस्ट्रॉय करें
        speechRecord.destroy()
    end,

    onError = function(errorCode)
        local errorMessages = {
            [SpeechRecognizer.ERROR_NETWORK] = "Network Error!",
            [SpeechRecognizer.ERROR_NO_MATCH] = "No Match Found!",
            [SpeechRecognizer.ERROR_SPEECH_TIMEOUT] = "Speech Timeout!",
            [SpeechRecognizer.ERROR_CLIENT] = "Client Error!"
        }
        local errorMsg = errorMessages[errorCode] or "Sorry! I Did Not Understand Your Message!"
        service.asyncSpeak(errorMsg)

        -- स्पीच रिकग्नाइज़र को डेस्ट्रॉय करें
        speechRecord.destroy()
    end
}

-- स्पीच लिस्निंग शुरू करें
speechRecord.startListening(recordIntent)